import json
from pathlib import Path
import hashlib
from collections import OrderedDict
from .bootstrap import SPY_RESULTS_DIR
from data_surveillance.bootstrap import HASH_OUTPUT_FILE_NAME


def hash_content(path: Path) -> dict:
    _tree = dict()
    for root, _, files in path.walk():
        root_path = Path(root)
        for file in files:
            if file.startswith("."):
                continue
            file_path = root_path / file
            _tree[str(f"{root_path.name}__{file}")] = str(
                hashlib.sha256(file_path.read_bytes()).digest(),
            )
    return {k: _tree[k] for k in sorted(_tree)}


def save(path: Path, tree: dict) -> None:
    path.joinpath(HASH_OUTPUT_FILE_NAME).write_text(json.dumps(tree))


def hash_walk(path: Path) -> None:
    tree = hash_content(path=path)
    save(path, tree)



import os
import shutil
from pathlib import Path
from typing import Generator

from pytest import fixture


@fixture
def path() -> Generator[Path]:
    path = Path(os.environ["_SPY_RESULTS_DIR"])
    path.mkdir(exist_ok=True)
    yield path

    shutil.rmtree(path)



import json
from pathlib import Path

from data_surveillance.bootstrap import HASH_OUTPUT_FILE_NAME


def find_miss(v1: dict, v2: dict) -> set[str]:
    diff = set(v1) - set(v2)
    for file in diff:
        print(f"miss - {file}")

    return diff


def find_diff(v1: dict, v2: dict) -> set[str]:
    intrsection = set(v1).intersection(set(v2))
    diff = set()

    for file in intrsection:
        if v1[file] != v2[file]:
            print(f"diff - {file}")
            diff.add(file)
    return diff


def diff(base: Path, test: Path) -> None:
    if base == test:
        raise ValueError("base and test paths can not be the same")

    if not all([base.exists(), test.exists()]):
        raise ValueError("base and test paths have to exist")

    if base.is_dir() and base.joinpath(HASH_OUTPUT_FILE_NAME).exists():
        base = base.joinpath(HASH_OUTPUT_FILE_NAME)
    else:
        raise ValueError("base file not found")

    if test.is_dir() and test.joinpath(HASH_OUTPUT_FILE_NAME).exists():
        test = test.joinpath(HASH_OUTPUT_FILE_NAME)
    else:
        raise ValueError("test file not found")

    base_encoded: dict = json.loads(base.read_text())
    test_encoded: dict = json.loads(test.read_text())

    print("Base vs Test - miss")
    find_miss(base_encoded, test_encoded)
    print("-" * 10)
    print("Test vs Base - miss")
    find_miss(test_encoded, base_encoded)
    print("-" * 10)
    print("Diff")
    find_diff(base_encoded, test_encoded)
    print("-" * 10)



from pathlib import Path
import click
from data_surveillance import hash_walk
from data_surveillance import diff as _diff


@click.group()
def cli():
    pass


@cli.command()
@click.argument(
    "path",
    type=click.Path(
        exists=True,
        file_okay=False,
        readable=True,
        path_type=Path,
    ),
)
def walk(path: Path):
    hash_walk(Path(path))


@cli.command()
@click.argument(
    "base",
    type=click.Path(
        exists=True,
        file_okay=False,
        readable=True,
        path_type=Path,
    ),
)
@click.argument(
    "test",
    type=click.Path(
        exists=True,
        file_okay=False,
        readable=True,
        path_type=Path,
    ),
)
def diff(base: Path, test: Path):
    _diff(base, test)



import os
from pathlib import Path
from random import randint

if _path := os.environ.get("_SPY_RESULTS_DIR"):
    SPY_RESULTS_DIR = Path(_path)
else:
    print(Path.cwd())
    SPY_RESULTS_DIR = Path.cwd() / f".{randint(1000, 9999)}"
    print(SPY_RESULTS_DIR)


SPY_RESULTS_DIR = Path(os.environ["_SPY_RESULTS_DIR"])
HASH_OUTPUT_FILE_NAME: str = ".hashed.json"



import json
import os
from pathlib import Path
import pickle
from typing import Any, Callable, Iterable, Literal

import pandas as pd
import logging
from logging import StreamHandler
from .bootstrap import SPY_RESULTS_DIR

logger = logging.getLogger(__name__)
logger.addHandler(StreamHandler())
logger.setLevel(logging.INFO)


def _save(
    root_folder: Path,
    step: int,
    value: Any,
    key: None | str = None,
    opt_type: Literal["in", "out", ""] = "in",
    file_name: str = "",
) -> None:
    _step = f"0{step}" if step < 10 else str(step)

    file_name += f"{opt_type}__{_step}__type={type(value).__name__}"

    if key:
        file_name = f"{file_name}__key={key}"

    if isinstance(value, (pd.DataFrame, pd.Series)):
        file_name += ".csv"
        value.to_csv(root_folder.joinpath(file_name))
    elif any([isinstance(value, list), isinstance(value, tuple)]) and any(
        [
            isinstance(value[0], pd.DataFrame),
            isinstance(value[0], pd.Series),
        ]
    ):
        for i, element in enumerate(value):
            _save(root_folder, i, element, None, "", file_name)
    else:
        try:
            to_file = json.dumps(value)
            file_name += ".json"
        except TypeError as ex:
            logger.info(ex.args[0])
            to_file = str(pickle.dumps(value))
            file_name += ".pickle"

        with root_folder.joinpath(file_name).open("w") as f:
            f.writelines(to_file)


def spy(func) -> Callable:
    path = SPY_RESULTS_DIR

    def inner(*arg, **kwargs):  # noqa: ANN002, ANN003, ANN202
        counter_file = path / ".counter"
        if counter_file.exists():
            step = counter_file.read_text()
            step = "0" if step == "" else str(int(step) + 1)
            counter_file.write_text(step)
        else:
            step = "0"
            counter_file.write_text(step)

        _step = f"0{step}" if int(step) < 10 else str(step)

        step_folder = path.joinpath(f"step_{_step}__{func.__qualname__}")
        step_folder.mkdir(exist_ok=True)

        small_step = 0
        for v in arg:
            _save(step_folder, small_step, v, None, "in")
            small_step += 1

        for k, v in kwargs.items():
            _save(step_folder, small_step, v, k, "in")
            small_step += 1

        r = func(*arg, **kwargs)
        if r is None:
            return None

        if isinstance(r, (list, tuple)):
            for _ in r:
                _save(step_folder, small_step, _, None, "out")
                small_step += 1
        else:
            _save(step_folder, small_step, r, None, "out")
        return r

    return inner


# # TODO: catch input output X
# # TODO: do numbers {00, 01} not {0,1} X
# # TODO: add other python objects X
# # TODO: Add hash comaprison X

# # TODO: add script which adding to every function my decorator:) -> orders with @staticfunction



import json
import os
from pathlib import Path
from typing import Any
from data_surveillance.spy import spy
import pytest
import pandas as pd


def get_all_genereated_files(path: Path) -> list[Path]:
    return sorted(
        [
            root.joinpath(file)
            for (root, _, files) in path.walk(top_down=False)
            for file in files
            if not file.startswith(".")
        ]
    )


def test_spy_in_one_func(path: Path):
    # given
    value = 1

    @spy
    def calc(value: int):
        pass

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 1, "File wasn't created"


def test_spy_in_two_func(path: Path):
    # given
    value = 1

    @spy
    def calc(value: int):
        pass

    # when
    calc(value=value)
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 2


def test_spy_in_out_one_func(path: Path) -> None:
    # given
    value = 1

    @spy
    def calc(value: int):
        return value * 10

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 2, "Files weren't created"


def test_spy_in_out_many_func(path: Path) -> None:
    # given
    value = 1

    @spy
    def calc(value: int):
        return value * 10

    # when
    calc(value=value)
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 4, "Files weren't created"


def test_spy_in_many_one_func(path: Path):
    # given
    value = 1

    @spy
    def calc(value: int, value1: int, value2: int):
        pass

    # when
    calc(value=value, value1=value, value2=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 3, "Files weren't created"


def test_spy_in_many_out_many_one_func(path: Path):
    # given
    value = 1

    @spy
    def calc(value: int, value1: int, value2: int):
        return (value, value1, value2)

    # when
    calc(value=value, value1=value, value2=value)

    # then
    files = get_all_genereated_files(path=path)
    assert len(files) == 6, "Files weren't created"


def test_spy_in_one_int_one_func(path: Path):
    # given
    _int = 1

    @spy
    def calc(_i: int):
        pass

    # when
    calc(_i=_int)

    # then
    files = get_all_genereated_files(path=path)
    assert str(_int) in files[0].read_text()


def test_spy_in_one_str_one_func(path: Path):
    # given
    value: str = "test"

    @spy
    def calc(value: str):
        pass

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert str(value) in files[0].read_text()


def test_spy_in_one_tuple_one_func(path: Path):
    # given
    value = (1, "2")

    @spy
    def calc(value):
        pass

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert list(value) == json.loads(files[0].read_text())


def test_spy_in_one_list_func(path: Path):
    # given
    value = [1, "2"]

    @spy
    def calc(value):
        pass

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert value == json.loads(files[0].read_text())


def test_spy_in_one_dict_one_func(path: Path):
    # given
    value = {"one": 1, "two": "2"}

    @spy
    def calc(value):
        pass

    # when
    calc(value=value)

    # then
    files = get_all_genereated_files(path=path)
    assert value == json.loads(files[0].read_text())


def test_spy_in_one_dataframe_one_func(path: Path) -> None:
    # given
    value = pd.DataFrame(
        {
            "col_1": [1, 2, 3],
            "col_2": [11, 22, 33],
        },
    )

    @spy
    def calc(value):
        pass

    # when
    calc(value=value)
    # then
    files = get_all_genereated_files(path=path)
    pd.testing.assert_frame_equal(value, pd.read_csv(files[0], index_col=0))


def test_spy_out_one_dataframe_one_func(path: Path) -> None:
    # given
    value = pd.DataFrame(
        {
            "col_1": [1, 2, 3],
            "col_2": [11, 22, 33],
        },
    )

    @spy
    def calc():
        return value

    # when
    calc()
    # then
    files = get_all_genereated_files(path=path)
    pd.testing.assert_frame_equal(value, pd.read_csv(files[0], index_col=0))


def test_spy_in_one_list_of_dataframes_one_func(path: Path) -> None:
    # given
    df1 = pd.DataFrame(
        {
            "col_1": [1, 2, 3],
            "col_2": [11, 22, 33],
        },
    )

    df2 = pd.DataFrame(
        {
            "col_2": [1, 2, 3],
            "col_3": [11, 22, 33],
        },
    )

    @spy
    def calc(value: list[pd.DataFrame]):
        pass

    # when
    calc(value=[df1, df2])
    # then
    files = get_all_genereated_files(path=path)
    pd.testing.assert_frame_equal(df1, pd.read_csv(files[0], index_col=0))
    pd.testing.assert_frame_equal(df2, pd.read_csv(files[1], index_col=0))


def test_spy_out_one_list_of_dataframes_one_func(path: Path) -> None:
    # given
    df1 = pd.DataFrame(
        {
            "col_1": [1, 2, 3],
            "col_2": [11, 22, 33],
        },
    )

    df2 = pd.DataFrame(
        {
            "col_2": [1, 2, 3],
            "col_3": [11, 22, 33],
        },
    )

    @spy
    def calc():
        return [df1, df2]

    # when
    calc()
    # then
    files = get_all_genereated_files(path=path)
    pd.testing.assert_frame_equal(df1, pd.read_csv(files[0], index_col=0))
    pd.testing.assert_frame_equal(df2, pd.read_csv(files[1], index_col=0))


def test_spy_in_one_series_one_func(path: Path) -> None:
    # given
    value = pd.Series(data=[1, 2, 3], name="0")

    @spy
    def calc(value):
        pass

    # when
    calc(value=value)
    # then
    files = get_all_genereated_files(path=path)
    pd.testing.assert_series_equal(value, pd.read_csv(files[0], index_col=0)["0"])



import hashlib
from pathlib import Path
from data_surveillance.hash_walk import hash_content
import pytest


def test_hash_walk_one_file(path: Path) -> None:
    # given
    value = b"test"
    file_name = "file.txt"

    # when
    path.joinpath(file_name).write_bytes(value)
    result: dict = hash_content(path)
    # then

    assert len(result) == 1
    given_key, given_value = result.popitem()
    assert file_name in given_key
    assert given_value == str(hashlib.sha256(value).digest())


def test_hash_walk_many_files(path: Path) -> None:
    # given
    value1 = b"test1"
    value2 = b"test2"
    file_name1 = "file1.txt"
    file_name2 = "file2.txt"

    # when
    path.joinpath(file_name1).write_bytes(value1)
    path.joinpath(file_name2).write_bytes(value2)
    result: dict = hash_content(path)
    # then

    assert len(result) == 2

    given_key, given_value = result.popitem()
    assert file_name2 in given_key
    assert given_value == str(hashlib.sha256(value2).digest())

    given_key, given_value = result.popitem()
    assert file_name1 in given_key
    assert given_value == str(hashlib.sha256(value1).digest())
